@@ -1,4 +1,4 @@
-pBI <- function(dataset, classlabels, referenceclasslabel, ids, useMedian = TRUE, lambda = 100, plotScores = TRUE, numTopRankedToPlot = 10)
+pBI <- function(dataset, classlabels, referenceclasslabel, ids, useMedian = TRUE, lambda = 100, plotScores = TRUE, numTopRankedToPlot = 10, bars.cols = c("red","blue"))
 {
 	if(!is.matrix(dataset))
 		stop("Dataset must be a matrix!")
@@ -41,11 +41,11 @@ pBI <- function(dataset, classlabels, referenceclasslabel, ids, useMedian = TRUE
 	 })
 	 names(pBIScores) <- rownames(dataset)
 	if(plotScores)
-		.plotScores(pBIScores, numTopRankedToPlot, method="pBI")
+		.plotScores(pBIScores, numTopRankedToPlot, method="pBI",  bars.cols = bars.cols)
 	return(pBIScores)
 }
 
-uBI <- function(dataset, classlabels, referenceclasslabel, useMedian = TRUE, lambda = 100, plotScores = TRUE, numTopRankedToPlot = 10)
+uBI <- function(dataset, classlabels, referenceclasslabel, useMedian = TRUE, lambda = 100, plotScores = TRUE, numTopRankedToPlot = 10, bars.cols = c("red","blue"))
 {
 	if(!is.matrix(dataset))
 		stop("Dataset must be a matrix!")
@@ -57,10 +57,8 @@ uBI <- function(dataset, classlabels, referenceclasslabel, useMedian = TRUE, lam
 		stop("Reference classlabel must be a valid classlabel!")
 	if(sum(is.na(dataset))> 0)
 		stop("No missing values are allowed!")
-	if(any(dataset==0))
-		stop("No zero values are allowed!")
 	uBIScores <- apply(dataset, 1, function(x){
-		tp2 <- .getTP2(x, classlabels)
+		tp2 <- .getTP2(x, classlabels, referenceclasslabel)
 		x.ref <- x[classlabels==referenceclasslabel]
 		x.comp <- x[classlabels!=referenceclasslabel]
 		if(useMedian){
@@ -88,33 +86,38 @@ uBI <- function(dataset, classlabels, referenceclasslabel, useMedian = TRUE, lam
 	 })
 	 names(uBIScores) <- rownames(dataset)
 	if(plotScores)
-		.plotScores(uBIScores, numTopRankedToPlot, method="uBI")
+		.plotScores(uBIScores, numTopRankedToPlot, method="uBI", bars.cols = bars.cols)
 	return(uBIScores)
 }
 
-.plotScores <- function(scores, numTopRankedToPlot, method)
+.plotScores <- function(scores, numTopRankedToPlot, method, bars.cols = c("red","blue"))
 {
 		numTopRanked <- min(numTopRankedToPlot, length(scores))
 		scoresRanked <- scores[match(sort(abs(scores), decreasing = TRUE), abs(scores))]
 		scoresTopRanked <- scoresRanked[1:numTopRanked]
-		cols <- ifelse(scoresTopRanked >= 0, "red", "blue")
+		cols <- ifelse(scoresTopRanked >= 0, bars.cols[1], bars.cols[2])
 		zoom <- 1.5
 		barplot(scoresTopRanked, main =paste(method, " scores (top ", numTopRanked, "ranked attributes)", sep=" "), ylab=method, col=cols, cex.axis = zoom, cex.names=zoom, cex.lab=zoom, cex.main=zoom)  
 }
 
-.getTP2<- function (values, classlabels) {
-		 tmp <- as.data.frame(cbind(values, classlabels))
-		 tmp$classlabels <- as.factor( tmp$classlabels)
-		 model <- glm(classlabels~values, data=tmp, family=binomial(link = "logit"))
-		confMat <- table(fitted(model)> 0.5, tmp$classlabels)
-		tp2 <- 1
-		for(i in 1:length(unique(classlabels)))
-		{
-			tp <- (diag(confMat)/apply(confMat, 2, sum))[i]
-			if(tp < 0.5)
-				tp <- 0
-			tp2 <- tp2 * tp
-		}
+.getTP2<- function (values, classlabels, referenceclasslabel) {
+		tmp <- data.frame(values,  classlabels=as.factor(classlabels))
+		model <- glm(classlabels~values, data=tmp, family=binomial(link = "logit"))
+		predictions <- as.numeric(fitted(model) > 0.5)+1   
+		predictions[predictions==1] <- levels(tmp$classlabels)[1]
+		predictions[predictions==2] <- levels(tmp$classlabels)[2]
+		positive.label <- unique(classlabels)[unique(classlabels)!= referenceclasslabel]
+		negative.label <- unique(classlabels)[unique(classlabels)== referenceclasslabel]
+	
+		TP <- sum(classlabels == positive.label & predictions == positive.label)
+		TN <- sum(classlabels == negative.label & predictions == negative.label)
+		FP <- sum(classlabels == negative.label & predictions == positive.label)
+		FN <- sum(classlabels == positive.label & predictions == negative.label)
+		sensitivity  <- TP / (TP + FN)
+		specificity <- TN / (FP + TN)
+		if(sensitivity < 0.5 | specificity < 0.5)
+			return(0)
+		tp2 <- sensitivity*specificity
 		tp2 <- ((tp2-0.25)*4/3)
 		return(tp2)		
 }
